@@ -16,11 +16,11 @@
 # limitations under the License.
 
 import struct
-import itertools
 
 from ryu import exception
 from ryu.lib import mac
 from ryu.lib.pack_utils import msg_pack_into
+from . import ofproto_parser
 from . import ofproto_v1_0
 from . import inet
 
@@ -63,7 +63,7 @@ _MF_FIELDS = {}
 FLOW_N_REGS = 8  # ovs 1.5
 
 
-class Flow(object):
+class Flow(ofproto_parser.StringifyMixin):
     def __init__(self):
         self.in_port = 0
         self.dl_vlan = 0
@@ -93,7 +93,7 @@ class Flow(object):
         self.pkt_mark = 0
 
 
-class FlowWildcards(object):
+class FlowWildcards(ofproto_parser.StringifyMixin):
     def __init__(self):
         self.dl_src_mask = 0
         self.dl_dst_mask = 0
@@ -115,13 +115,29 @@ class FlowWildcards(object):
         self.pkt_mark_mask = 0
 
 
-class ClsRule(object):
+class ClsRule(ofproto_parser.StringifyMixin):
     """describe a matching rule for OF 1.0 OFPMatch (and NX).
     """
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.wc = FlowWildcards()
         self.flow = Flow()
 
+        for key, value in kwargs.items():
+            if key[:3] == 'reg':
+                register = int(key[3:] or -1)
+                self.set_reg(register, value)
+                continue
+
+            setter = getattr(self, 'set_' + key, None)
+            if not setter:
+                LOG.error('Invalid kwarg specified to ClsRule (%s)', key)
+                continue
+
+            if not isinstance(value, (tuple, list)):
+                value = (value, )
+
+            setter(*value)
+
     def set_in_port(self, port):
         self.wc.wildcards &= ~FWW_IN_PORT
         self.flow.in_port = port
@@ -917,14 +933,16 @@ class MFRegister(MFField):
 
 
 @_register_make
-@_set_nxm_headers([ofproto_v1_0.NXM_NX_PKT_MARK, ofproto_v1_0.NXM_NX_PKT_MARK_W])
+@_set_nxm_headers([ofproto_v1_0.NXM_NX_PKT_MARK,
+                   ofproto_v1_0.NXM_NX_PKT_MARK_W])
 class MFPktMark(MFField):
     @classmethod
     def make(cls, header):
         return cls(header, MF_PACK_STRING_BE32)
 
     def put(self, buf, offset, rule):
-        return self.putm(buf, offset, rule.flow.pkt_mark, rule.wc.pkt_mark_mask)
+        return self.putm(buf, offset, rule.flow.pkt_mark,
+                         rule.wc.pkt_mark_mask)
 
 
 def serialize_nxm_match(rule, buf, offset):
