@@ -666,9 +666,9 @@ OFPGC_REMOVE_BUCKET = 5     # Remove all action buckets or any specific action
                             # bucket from matching group
 
 # enum ofp_group_bucket_prop_type
-OFPGBPT_WEIGHT = 0      # Select groups only.
-OFPGBPT_WATCH_PORT = 1  # Fast failover groups only.
-OFPGBPT_WATCH_GROU = 2  # Fast failover groups only.
+OFPGBPT_WEIGHT = 0             # Select groups only.
+OFPGBPT_WATCH_PORT = 1         # Fast failover groups only.
+OFPGBPT_WATCH_GROUP = 2        # Fast failover groups only.
 OFPGBPT_EXPERIMENTER = 0xFFFF  # Experimenter defined.
 
 # struct ofp_group_bucket_prop_header
@@ -745,9 +745,10 @@ OFPGT_FF = 3        # Fast failover group.
 OFP_NO_BUFFER = 0xffffffff  # Special buffer-id to indicate 'no buffer'
 
 # struct ofp_packet_out
-OFP_PACKET_OUT_PACK_STR = '!IH2x'
+OFP_PACKET_OUT_0_PACK_STR = '!IH2x'
+OFP_PACKET_OUT_0_SIZE = 16
 OFP_PACKET_OUT_SIZE = 24
-assert (calcsize(OFP_PACKET_OUT_PACK_STR) + OFP_MATCH_SIZE + OFP_HEADER_SIZE ==
+assert (calcsize(OFP_PACKET_OUT_0_PACK_STR) + OFP_MATCH_SIZE + OFP_HEADER_SIZE ==
         OFP_PACKET_OUT_SIZE)
 
 # enum ofp_packet_in_reason
@@ -1197,7 +1198,7 @@ assert (calcsize(OFP_FLOW_STATS_REQUEST_PACK_STR) ==
         OFP_FLOW_STATS_REQUEST_SIZE)
 
 # struct ofp_flow_desc
-_OFP_FLOW_DESC_0_PACK_STR = 'H2xBBHHHHHQ'
+_OFP_FLOW_DESC_0_PACK_STR = 'H2xBxHHHHHQ'
 OFP_FLOW_DESC_0_PACK_STR = '!' + _OFP_FLOW_DESC_0_PACK_STR
 OFP_FLOW_DESC_0_SIZE = 24
 assert calcsize(OFP_FLOW_DESC_0_PACK_STR) == OFP_FLOW_DESC_0_SIZE
@@ -1375,11 +1376,6 @@ OFP_BUCKET_COUNTER_PACK_STR = '!QQ'
 OFP_BUCKET_COUNTER_SIZE = 16
 assert calcsize(OFP_BUCKET_COUNTER_PACK_STR) == OFP_BUCKET_COUNTER_SIZE
 
-# struct ofp_group_desc_stats
-OFP_GROUP_DESC_STATS_PACK_STR = '!HBxI'
-OFP_GROUP_DESC_STATS_SIZE = 8
-assert calcsize(OFP_GROUP_DESC_STATS_PACK_STR) == OFP_GROUP_DESC_STATS_SIZE
-
 # struct ofp_group_stats
 OFP_GROUP_STATS_PACK_STR = '!H2xII4xQQII'
 OFP_GROUP_STATS_SIZE = 40
@@ -1390,6 +1386,12 @@ OFP_GROUP_DESC_PACK_STR = '!HBxIH6x'
 OFP_GROUP_DESC_SIZE = 16
 assert calcsize(OFP_GROUP_DESC_PACK_STR) == OFP_GROUP_DESC_SIZE
 
+# struct ofp_group_desc_stats
+# Backward compatibility with 1.3.1 - avoid breaking the API.
+OFP_GROUP_DESC_STATS_PACK_STR = OFP_GROUP_DESC_PACK_STR
+OFP_GROUP_DESC_STATS_SIZE = OFP_GROUP_DESC_SIZE
+assert calcsize(OFP_GROUP_DESC_STATS_PACK_STR) == OFP_GROUP_DESC_STATS_SIZE
+
 # enum ofp_group_capabilities
 OFPGFC_SELECT_WEIGHT = 1 << 0       # Support weight for select groups.
 OFPGFC_SELECT_LIVENESS = 1 << 1     # Support liveness for select groups.
@@ -1628,14 +1630,11 @@ OFP_TIME_SIZE = 16
 assert calcsize(OFP_TIME_PACK_STR) == OFP_TIME_SIZE
 
 # struct ofp_bundle_features_prop_time
-OFP_BUNDLE_FEATURES_PROP_TIME_PACK_STR = ('!HH4x' +
-                                          _OFP_TIME_PACK_STR +
-                                          _OFP_TIME_PACK_STR +
-                                          _OFP_TIME_PACK_STR +
-                                          _OFP_TIME_PACK_STR)
+OFP_BUNDLE_FEATURES_PROP_TIME_0_PACK_STR = '!HH4x'
+OFP_BUNDLE_FEATURES_PROP_TIME_0_SIZE = 8
 OFP_BUNDLE_FEATURES_PROP_TIME_SIZE = 72
-assert (calcsize(OFP_BUNDLE_FEATURES_PROP_TIME_PACK_STR) ==
-        OFP_BUNDLE_FEATURES_PROP_TIME_SIZE)
+assert (calcsize(OFP_BUNDLE_FEATURES_PROP_TIME_0_PACK_STR) +
+        OFP_TIME_SIZE * 4 == OFP_BUNDLE_FEATURES_PROP_TIME_SIZE)
 
 # enum ofp_bundle_feature_flags
 OFPBF_TIMESTAMP = 1 << 0        # Request includes a timestamp.
@@ -1810,6 +1809,15 @@ OFP_BUNDLE_ADD_MSG_SIZE = 24
 assert (calcsize(OFP_BUNDLE_ADD_MSG_PACK_STR) + OFP_HEADER_SIZE ==
         OFP_BUNDLE_ADD_MSG_SIZE)
 
+# Note: struct ofp_prop_experimenter is specific to this implementation.
+# It does not have a corresponding structure in the specification.
+# This structure defines common structure for ofp_*_prop_experimenter.
+# struct ofp_prop_experimenter
+OFP_PROP_EXPERIMENTER_PACK_STR = '!HHII'
+OFP_PROP_EXPERIMENTER_SIZE = 12
+assert (calcsize(OFP_PROP_EXPERIMENTER_PACK_STR) ==
+        OFP_PROP_EXPERIMENTER_SIZE)
+
 # define constants
 OFP_VERSION = 0x06
 OFP_TCP_PORT = 6653
