@@ -21,6 +21,7 @@ import math
 import netaddr
 import os
 import signal
+import six
 import sys
 import time
 import traceback
@@ -30,13 +31,13 @@ from ryu import cfg
 
 # import all packet libraries.
 PKT_LIB_PATH = 'ryu.lib.packet'
-for modname, moddef in sys.modules.iteritems():
+for modname, moddef in sys.modules.items():
     if not modname.startswith(PKT_LIB_PATH) or not moddef:
         continue
     for (clsname, clsdef, ) in inspect.getmembers(moddef):
         if not inspect.isclass(clsdef):
             continue
-        exec 'from %s import %s' % (modname, clsname)
+        exec('from %s import %s' % (modname, clsname))
 
 from ryu.base import app_manager
 from ryu.controller import handler
@@ -51,6 +52,7 @@ from ryu.ofproto import ofproto_protocol
 from ryu.ofproto import ofproto_v1_3
 from ryu.ofproto import ofproto_v1_3_parser
 from ryu.ofproto import ofproto_v1_4
+from ryu.ofproto import ofproto_v1_5
 
 
 """ Required test network:
@@ -287,13 +289,15 @@ class OfTester(app_manager.RyuApp):
         def __get_version(opt):
             vers = {
                 'openflow13': ofproto_v1_3.OFP_VERSION,
-                'openflow14': ofproto_v1_4.OFP_VERSION
+                'openflow14': ofproto_v1_4.OFP_VERSION,
+                'openflow15': ofproto_v1_5.OFP_VERSION
             }
             ver = vers.get(opt.lower())
             if ver is None:
                 self.logger.error(
                     '%s is not supported. '
-                    'Supported versions are openflow13 and openflow14.',
+                    'Supported versions are openflow13, '
+                    'openflow14 and openflow15.',
                     opt)
                 self._test_end()
             return ver
@@ -360,7 +364,8 @@ class OfTester(app_manager.RyuApp):
     def _register_sw(self, dp):
         vers = {
             ofproto_v1_3.OFP_VERSION: 'openflow13',
-            ofproto_v1_4.OFP_VERSION: 'openflow14'
+            ofproto_v1_4.OFP_VERSION: 'openflow14',
+            ofproto_v1_5.OFP_VERSION: 'openflow15'
         }
         if dp.id == self.target_dpid:
             if dp.ofproto.OFP_VERSION != OfTester.target_ver:
@@ -745,7 +750,7 @@ class OfTester(app_manager.RyuApp):
         if after_tester_send == before_tester_send:
             log_msg = 'no change in tx_packets on tester.'
         elif after_target_receive == before_target_receive:
-            log_msg = 'no change in rx_packtes on target.'
+            log_msg = 'no change in rx_packets on target.'
         elif test_type == KEY_EGRESS:
             if after_target_send == before_target_send:
                 log_msg = 'no change in tx_packets on target.'
@@ -878,7 +883,7 @@ class OfTester(app_manager.RyuApp):
                             'ipv6_flabel': 20,
                             'ipv6_exthdr': 9}
             match_fields = list()
-            for key, united_value in match.iteritems():
+            for key, united_value in match.items():
                 if isinstance(united_value, tuple):
                     (value, mask) = united_value
                     # look up oxm_fields.TypeDescr to get mask length.
@@ -1304,7 +1309,7 @@ class TestFile(stringify.StringifyMixin):
                 if v[k] == port_name:
                     v[k] = CONF['test-switch'][port_name]
         if isinstance(val, dict):
-            for k, v in val.iteritems():
+            for k, v in val.items():
                 if k == "OFPActionOutput":
                     if 'port' in v:
                         __replace_port_name("port", v)
@@ -1317,19 +1322,19 @@ class TestFile(stringify.StringifyMixin):
                 self._normalize_test_json(v)
 
     def _get_tests(self, path):
-        with open(path, 'rb') as fhandle:
+        with open(path, 'r') as fhandle:
             buf = fhandle.read()
             try:
                 json_list = json.loads(buf)
                 for test_json in json_list:
-                    if isinstance(test_json, unicode):
+                    if isinstance(test_json, six.text_type):
                         self.description = test_json
                     else:
                         self._normalize_test_json(test_json)
                         self.tests.append(Test(test_json))
             except (ValueError, TypeError) as e:
                 result = (TEST_FILE_ERROR %
-                          {'file': path, 'detail': e.message})
+                          {'file': path, 'detail': str(e)})
                 self.logger.warning(result)
 
 
@@ -1344,7 +1349,7 @@ class Test(stringify.StringifyMixin):
         def __test_pkt_from_json(test):
             data = eval('/'.join(test))
             data.serialize()
-            return str(data.data)
+            return six.binary_type(data.data)
 
         def __normalize_match(ofproto, match):
             match_json = match.to_jsondict()
