@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import logging
+from ryu.controller import handler
 from ryu.controller import event
 
 LOG = logging.getLogger(__name__)
@@ -149,3 +150,19 @@ class EventHostReply(event.EventReplyBase):
     def __str__(self):
         return 'EventHostReply<dst=%s, dpid=%s, hosts=%s>' % \
             (self.dst, self.dpid, len(self.hosts))
+
+
+class EventHostBase(event.EventBase):
+    def __init__(self, host):
+        super(EventHostBase, self).__init__()
+        self.host = host
+
+    def __str__(self):
+        return '%s<%s>' % (self.__class__.__name__, self.host)
+
+
+class EventHostAdd(EventHostBase):
+    def __init__(self, host):
+        super(EventHostAdd, self).__init__(host)
+
+handler.register_service('ryu.topology.switches')
